package com.dashitech.migration.core;

import com.dashitech.migration.util.JdbcRow;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 字段匹配器
 * Created by CX on 2024/10/10.
 */
public class FieldMapper {

    /**
     * 待匹配字段
     */
    private List<DataMapper> fieldList = new ArrayList<>();
    private Map<String, Long> sourceCache = new HashMap<>();
    private Map<String, Long> targetCache = new HashMap<>();
    /**
     * 匹配完成后的id对照，source:target
     */
    private Map<Long, Long> idCache = new HashMap<>();
    /**
     * 待转换的插入数据
     */
    private List<JdbcRow> rows = new ArrayList<>();
    /**
     * 插入数据，需要替换或删除的字段
     */
    private List<DataMapper> rowsOperationList = new ArrayList<>();

    /**
     * 根据rowsOperationList ,处理成可以直接插入的最终数据
     *
     * @param list 原始数据：List<Map<String,Object>> 类型
     */
    public void trans2JdbcRow(List list) {
        if (list != null && list.size() > 0) {
            for (Object object : list) {
                Map<String, Object> row = (Map<String, Object>) object;

                for (DataMapper dataMapper : rowsOperationList) {
                    //如果是删除，直接删除旧key
                    if (dataMapper.isRemove() && row.containsKey(dataMapper.getSourceName())) {
                        row.remove(dataMapper.getSourceName());
                    }
                    //如果是修改，先删除旧的key，在新增新的key:value
                    if (dataMapper.isUpdate() && row.containsKey(dataMapper.getSourceName())) {
                        Object value = row.get(dataMapper.getSourceName());
                        row.remove(dataMapper.getSourceName());
                        row.put(dataMapper.getTargetName(), value);
                    }
                    //如果是新增，直接put
                    if (dataMapper.isAdd()) {
                        row.put(dataMapper.getSourceName(), dataMapper.getTargetValue());
                    }
                    //如果是要替换的，使用id缓存替换
                    if (dataMapper.isReplace() && row.containsKey(dataMapper.getSourceName())) {
                        Object value = row.get(dataMapper.getSourceName());
                        if (value != null) {
                            Long id = dataMapper.getIdCache().get(value);
                            row.put(dataMapper.getSourceName(), id);
                        } else { //没有拿到数据，就置空
                            row.put(dataMapper.getSourceName(), null);
                        }
                    }
                }

                //判重复

                JdbcRow jdbcRow = row2JdbcRow(row);
                rows.add(jdbcRow);
            }
        }
    }

    private JdbcRow row2JdbcRow(Map<String, Object> row) {
        JdbcRow jdbcRow = new JdbcRow();
        jdbcRow.setRow(row);
        return jdbcRow;
    }


    /**
     * 增加待匹配字段，形成联合主键key
     */
    public FieldMapper addFieldMapper(String target, String source) {
        fieldList.add(new DataMapper(target, source));
        return this;
    }

    /**
     * 待删除的字段
     */
    public FieldMapper addOperationRemove(String name) {
        DataMapper mapper = new DataMapper(name, name);
        mapper.setRemove(true);
        rowsOperationList.add(mapper);
        return this;
    }

    /**
     * 待修改的字段
     */
    public FieldMapper addOperationUpdate(String target, String source) {
        DataMapper mapper = new DataMapper(target, source);
        mapper.setUpdate(true);
        rowsOperationList.add(mapper);
        return this;
    }

    /**
     * 待新增的默认字段
     */
    public FieldMapper addOperationAdd(String name, Object value) {
        DataMapper mapper = new DataMapper(name, name);
        mapper.setAdd(true);
        mapper.setTargetValue(value);
        rowsOperationList.add(mapper);
        return this;
    }

    /**
     * 待替换的字段，每次加入都在队列最前面。注意：如果是loadCache过滤，使用target名称，如果是start转换，使用source名称
     */
    public FieldMapper addOperationReplace(String name, Map<Long, Long> idCache) {
        DataMapper mapper = new DataMapper(name, name);
        mapper.setReplace(true);
        mapper.setIdCache(idCache);
        //TODO 替换，必须放在第一个，避免被修改字段名称后，导致使用新名称的错误
        rowsOperationList.add(0, mapper);
        return this;
    }

    /**
     * 使用完显式清空，释放内存
     */
    public void clear() {
        this.fieldList.clear();
        this.targetCache.clear();
        this.targetCache.clear();
        this.idCache.clear();
        this.rows.clear();
        this.rowsOperationList.clear();
    }

    public List<DataMapper> getFieldList() {
        return fieldList;
    }

    public void setFieldList(List<DataMapper> fieldList) {
        this.fieldList = fieldList;
    }

    public Map<String, Long> getSourceCache() {
        return sourceCache;
    }

    public void setSourceCache(Map<String, Long> sourceCache) {
        this.sourceCache = sourceCache;
    }

    public Map<String, Long> getTargetCache() {
        return targetCache;
    }

    public void setTargetCache(Map<String, Long> targetCache) {
        this.targetCache = targetCache;
    }

    public Map<Long, Long> getIdCache() {
        return idCache;
    }

    public void setIdCache(Map<Long, Long> idCache) {
        this.idCache = idCache;
    }

    public List<JdbcRow> getRows() {
        return rows;
    }

    public void setRows(List<JdbcRow> rows) {
        this.rows = rows;
    }

    public List<DataMapper> getRowsOperationList() {
        return rowsOperationList;
    }

    public void setRowsOperationList(List<DataMapper> rowsOperationList) {
        this.rowsOperationList = rowsOperationList;
    }
}
