package com.dashitech.migration.util;

import com.dashitech.utils.CommonUtil;
import com.dashitech.utils.DecimalUtil;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.jdbc.core.BatchPreparedStatementSetter;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;


/**
 * 数据库批量操作工具类
 */
public class JdbcUtil {
    private static final Logger log = LoggerFactory.getLogger(JdbcUtil.class);
    private JdbcTemplate jdbcTemplate;

    public JdbcUtil(JdbcTemplate jdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
    }



    /**
     * 获取插入语句
     */
    private String getInsertSql(String table,List<JdbcRow> rows) {
        Map<String, Object> cols = new LinkedHashMap<>();

        //顺序覆盖参数和传值
        for (JdbcRow row : rows) {
            cols.putAll(row.getRow());
        }
        //整理数据，没有的传值，补null
        for (JdbcRow row : rows) {
            Map<String, Object> _row = row.getRow();
            for(Map.Entry<String,Object> entry : cols.entrySet()) {
                //没有这个参数的，补齐null值，要不然批量新增参数不对报错
                if(!_row.containsKey(entry.getKey())) {
                    _row.put(entry.getKey(), null);
                }
            }
        }

        String result = "";
        StringBuffer sb = new StringBuffer();
        // 设置执行语句
        sb.append("insert into " + table + "( ");
        for (Map.Entry<String, Object> entry : cols.entrySet()) {
            sb.append(entry.getKey()).append(",");
        }
        if (sb.toString().endsWith(",")) {
            sb = sb.delete(sb.length() - 1, sb.length());
        }
        sb.append(" ) values( ");
        for (Map.Entry<String, Object> entry : cols.entrySet()) {
            sb.append("?,");
        }
        if (sb.toString().endsWith(",")) {
            sb = sb.delete(sb.length() - 1, sb.length());
        }
        sb.append(" )");    //批量语句，结尾不能加上分号;
        result = sb.toString();
        return result;
    }

    /**
     * 获取修改语句，必须带上条件！
     */
    private static String getUpdateSql(String table,JdbcRow row) {
        String result = "";
        StringBuffer sb = new StringBuffer();
        // 设置执行语句
        sb.append(" update " + table + " set ");
        for (Map.Entry<String, Object> entry : row.getRow().entrySet()) {
            sb.append(entry.getKey()).append(" = ? ").append(",");
        }
        if (sb.toString().endsWith(",")) {
            sb = sb.delete(sb.length() - 1, sb.length());
        }

        //设置条件语句
        sb.append(row.getWhere());
        //批量语句，结尾不能加上分号;
        result = sb.toString();
        return result;
    }


    /**
     * 批量执行修改数据
     */
    private boolean jdbcTemplateBatchUpdate(String sql, final List<JdbcRow> rows) {
        if (jdbcTemplate != null && StringUtils.isNotEmpty(sql) && rows != null && rows.size() > 0) {
            jdbcTemplate.batchUpdate(sql, new BatchPreparedStatementSetter() {
                public void setValues(PreparedStatement ps, int i) throws SQLException {
                    //设置数据参数
                    Map<String, Object> row = rows.get(i).getRow();
                    int index = 1;
                    for (Map.Entry<String, Object> entry : row.entrySet()) {
                        ps.setObject(index++, entry.getValue());
                    }

                    //设置where参数
                    List params = rows.get(i).getParams();
                    if (params.size() > 0) {
                        for (Object param : params) {
                            ps.setObject(index++, param);
                        }
                    }
                }

                public int getBatchSize() {
                    return rows == null ? 0 : rows.size();
                }
            });
            return true;
        }
        return false;
    }

    /**
     * 批量执行新增数据 <br/>
     * 新增不需要where条件
     * @param table        插入数据的表名
     * @param rows         行数据
     */
    private boolean batchInsert(String table, final List<JdbcRow> rows) {
        if (table == null || table.length()== 0 || rows == null || rows.size() == 0) {
            return false;
        }
        String insertSql = getInsertSql(table,rows);
        log.info("开始批量新增"+table+", 条数："+rows.size());
        return jdbcTemplateBatchUpdate(insertSql, rows);
    }

    /**
     * 批量执行修改数据 <br/>
     * 修改必须带上where条件
     * @param table        插入数据的表名
     * @param rows         行数据
     */
    private boolean batchUpdate(String table, final List<JdbcRow> rows) {
        if (table == null || table.length()== 0 || rows == null || rows.size() == 0) {
            return false;
        }
        String updateSql = getUpdateSql(table,rows.get(0));
        log.info("开始批量修改"+table+", 条数："+rows.size());
        return jdbcTemplateBatchUpdate(updateSql, rows);
    }

    /**
     * 返回分页后的列表
     * @param rows 总数据
     * @param page 一批要分为多少条
     */
    private static List<List<JdbcRow>> pageList(List<JdbcRow> rows,int page) {
        List<List<JdbcRow>> result = new ArrayList<List<JdbcRow>>();
        if(rows != null && rows.size() > page) {
            double div = Math.ceil(DecimalUtil.div(rows.size(),page).doubleValue());
            int total = Double.valueOf(div).intValue();

            for (int i = 1; i <= total; i++) {
                List<JdbcRow> tempList = CommonUtil.paginateList(rows, i, page);
                result.add(tempList);
            }
        }else if(rows != null && rows.size() > 0 && rows.size() <= page) {
            result.add(rows);
        }
        return result;
    }


    /**
     * 分批执行批量操作，默认1000条
     */
    public boolean batchPageInsert(String table, final List<JdbcRow> rows) {
        if (table == null || table.length()== 0 || rows == null || rows.size() == 0) {
            return false;
        }
        List<List<JdbcRow>> pageList = pageList(rows, 1000);
        for (List<JdbcRow> jdbcRows : pageList) {
            batchInsert(table, jdbcRows);
        }
        return true;
    }

    /**
     * 分批执行批量操作，默认1000条
     */
    public boolean batchPageUpdate(String table, final List<JdbcRow> rows) {
        if (table == null || table.length()== 0 || rows == null || rows.size() == 0) {
            return false;
        }
        List<List<JdbcRow>> pageList = pageList(rows, 1000);
        for (List<JdbcRow> jdbcRows : pageList) {
            batchUpdate(table, jdbcRows);
        }
        return true;
    }


    public <T> List<T> simpleSelectDto(String sql,Class<T> _class,Object... args) {
        return  jdbcTemplate.query(sql, new BeanPropertyRowMapper<>(_class), args);
    }

    /**
     * 返回列表的查询
     */
    public List simpleSelectList(String sql,Object... args) {
        return  jdbcTemplate.queryForList(sql, args);
    }

    public static void test() {
        List<JdbcRow> rows = new ArrayList<JdbcRow>();
        rows.add(JdbcRow.newInstance()
                .setAttribute("version", 1)
                .setAttribute("_name", "fafdsafsda")
                .setAttribute("_age", 1111)
//                .setWhere("id",30041)
        );

        rows.add(JdbcRow.newInstance()
                .setAttribute("version", 2)
                .setAttribute("_name", "BBBBB131232132132121BBBBBB")
                .setAttribute("_age", 22222)
//                .setWhere("id",30042)
        );

//        JdbcUtil.batchInsert("test",rows);
//        JdbcUtil.batchUpdate("test",rows);

    }


    public static void main(String[] args) {
        test();
    }

}
